package ActiveState::JournalDir;
use ActiveState::File::Atomic;
use File::Path;
use File::Glob qw(:glob);


use strict;
use warnings;
our $VERSION='1.000';


#################################
#      Exported functions       #   
#################################

sub new {
    my $class =shift;
    my $self = bless {}, ref($class) || $class;

    my $location = shift;

    my %args = (
	    create=>1, 
	    read_only=>0,
	    @_,
	    );

    if($args{create} && ! -d $location) {
	mkpath($location) || die "Failed to create $location";
    }

    $self->{directory} = $location;
    $self->{writable} = ($args{read_only} ? 0 : 1);

    return $self;
}


sub create{
    my $self = shift;
    my $entry_name = shift;

    my %opts = (
	    writable=>$self->{writable},
	    create=>1
	    );

    my $entry_subdir = $self->get_path($entry_name, 1);

    unless(-d $entry_subdir) {
	mkpath($entry_subdir) || die "Failed to create directory $entry_subdir.";
    }

    $self->{atomic}{$entry_name} =  
	ActiveState::File::Atomic->new($self->get_path($entry_name), %opts);

    if(defined($self->{atomic}{$entry_name})){
	return $self->{atomic}{$entry_name}->slurp;
    }
    else{
	return undef;
    }
}


sub fetch{
    my $self = shift;
    my $entry_name = shift;

    my %opts = (
	    writable=>$self->{writable},
	    create=>0
	    );
  
    if(-e $self->get_path($entry_name)) {
	unless($self->{atomic}{$entry_name}){
	    $self->{atomic}{$entry_name} =  
		ActiveState::File::Atomic->new($self->get_path($entry_name), %opts);
	}

	if($self->{atomic}{$entry_name} ){
	    return $self->{atomic}{$entry_name}->slurp;
	}
	else{
	    return undef;
	}
    }
    else{
	return undef;
    }
	
}


sub update{
    my $self = shift;
    my $entry_name = shift;
    my $new_entry = shift;

    if(!defined($self->{atomic}{$entry_name})){
	warn "There is no record for $entry_name in the journal";
    }
    elsif(!defined($new_entry)){
	warn "Null journal entry passed to update";
    }
    elsif(!$self->{writable})
    {
	warn "Not updating record due to read-only flag";

	$self->release($entry_name);
	$self->{atomic}{$entry_name} = undef;
    }
    else {
	$self->{atomic}{$entry_name}->commit_string($new_entry);
	$self->{atomic}{$entry_name} = undef;
    }
}


sub release{
    my $self = shift;
    my $entry_name = shift;
    my $silent = shift;


    if(!defined($self->{atomic}{$entry_name})){
	warn "There is no record for $entry_name in the journal" unless $silent;
    }
    else{
	$self->{atomic}{$entry_name}->close(); 
	$self->{atomic}{$entry_name} = undef;
    }
}


sub remove{
    my $self = shift;
    my $entry_name = shift;

    $self->release($entry_name, 1);
    unlink($self->get_path($entry_name));
}


sub list{
    my $self = shift;
    my @records;

    foreach my $dir(bsd_glob("$self->{directory}/*")){
        if(-d $dir){
            push(@records, bsd_glob("$dir/*"));  
        }
    }

    # rip the path off of the current entry
    foreach(@records){
        s/.*\///;
    }

    return @records;
}



#################################
#       Private functions       #   
#################################

sub get_path{
    my $self = shift;
    my $entry_name = shift;
    my $path_only = shift;

    my @name = split(//, $entry_name);

    my $path = "$self->{directory}/$name[0]$name[1]$name[$#name]";

    if($path_only){
	return $path;
    }
    else {
	return "$path/$entry_name"; 
    }
}


1;


=head1 NAME

JournalDir - manage a journal based on a directory containing a single file for
each journal entry

=head1 SYNOPSIS

    # read and update a journal entry
    my $jd = JournalDir->new('/tmp/jdir/test');

    my $data1 = $jd->fetch('entry1');

    my $now = localtime;
    $data1 .= "$now\n\n";
    $jd->update('entry1', $data1);


    # test the existence of and remove a journal entry
    my $jd = JournalDir->new('/tmp/jdir/test');

    $jd->fetch('entry1') and $jd->remove('entry1');;


=head1 METHODS

=over 4

=item new


 $jd = JournalDir->new("/base/journal/path", create=>1, read_only=>0)

Create a new JournalDir object, pointing to storage in "/base/journal/path".
Returns a new JournalDir or undef on failure;

Options are passed as key/value pairs after the filename.  The
supported options are:

=over 4

=item create

Create the directory tree named as the location (this is the default).  


=item read_only

Create the JournalDir object read-only (this is not the default). 


=back

=item create

 create("entry_id")

Create the named journal entry if it doesn't already exist, and returns the
journal entry as a string.  Returns undef if the record can't be created.
Locks the journal entry if one is found/created.

=item fetch

 fetch("entry_id")

Retrieves the named journal entry if it exists, and returns the journal entry
as a string.  Returns undef if the record can't be accessed.  Locks the journal
entry if one is found.

=item update

 fetch("entry_id", "new entry data")

Updates the specified journal entry, and unlocks it when done. 


=item release

 release("entry_id")

Unlocks a journal entry.  Does not remove the journal entry.


=item remove

remove("entry_id")

Removes an entry from the journal.


=item list

list()

Returns an array containing the the IDs of each journal entry.

=head1 KNOWN ISSUES

An entry that is deleted via 'remove' may be re-created if there is a pending
update to that record.

=cut
